"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const isWithinInterval_1 = __importDefault(require("date-fns/isWithinInterval"));
const toDate_1 = __importDefault(require("date-fns/toDate"));
const isEqual_1 = __importDefault(require("date-fns/isEqual"));
const parseISO_1 = __importDefault(require("date-fns/parseISO"));
const subMilliseconds_1 = __importDefault(require("date-fns/subMilliseconds"));
const addMilliseconds_1 = __importDefault(require("date-fns/addMilliseconds"));
const utils_1 = require("@terascope/utils");
const utils_2 = require("../../utils");
// TODO: handle datemath
function compareTermDates(node) {
    const nodeTermTime = convertDate(node.value, 0, true);
    return function dateTerm(date) {
        const result = convertDate(date, 0, false);
        if (!result)
            return false;
        return isEqual_1.default(result, nodeTermTime);
    };
}
exports.compareTermDates = compareTermDates;
function getRangeValues(node) {
    const rangeQuery = utils_2.parseRange(node);
    let incMin = rangeQuery.gte == null ? 1 : 0;
    let incMax = rangeQuery.lte == null ? -1 : 0;
    let minValue = rangeQuery.gte || rangeQuery.gt || '*';
    let maxValue = rangeQuery.lte || rangeQuery.lt || '*';
    if (utils_2.isInfiniteMin(minValue)) {
        incMin = 0;
        minValue = new Date(-8640000000000000);
    }
    if (utils_2.isInfiniteMax(maxValue)) {
        incMax = 0;
        maxValue = new Date(8640000000000000);
    }
    const start = convertDate(minValue, incMin, true);
    const end = convertDate(maxValue, incMax, true);
    return {
        start,
        end
    };
}
function dateRange(node) {
    const interval = getRangeValues(node);
    // verify it won't fail
    isWithinInterval_1.default(new Date(), interval);
    return function dateRangeTerm(date) {
        const result = convertDate(date, 0, false);
        if (!result)
            return false;
        try {
            return isWithinInterval_1.default(result, interval);
        }
        catch (err) {
            return false;
        }
    };
}
exports.dateRange = dateRange;
function convertDate(val, inclusive, throwErr) {
    const result = utils_1.getValidDate(val);
    if (result)
        return handleInclusive(result, inclusive);
    if (utils_1.isInteger(val))
        return handleInclusive(toDate_1.default(val), inclusive);
    if (utils_1.isString(val))
        return handleInclusive(parseISO_1.default(val), inclusive);
    if (throwErr)
        throw new Error(`Invalid date format ${val}`);
    return undefined;
}
function handleInclusive(date, inclusive) {
    if (inclusive === 0)
        return date;
    if (inclusive < 0)
        return subMilliseconds_1.default(date, inclusive);
    return addMilliseconds_1.default(date, inclusive);
}
//# sourceMappingURL=dates.js.map