"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const is_cidr_1 = __importDefault(require("is-cidr"));
// @ts-ignore TODO we should add types
const ip6addr_1 = __importDefault(require("ip6addr"));
const net_1 = require("net");
const utils_1 = require("../../utils");
const MIN_IPV4_IP = '0.0.0.0';
const MAX_IPV4_IP = '255.255.255.255';
const MIN_IPV6_IP = '::';
const MAX_IPV6_IP = 'ffff.ffff.ffff.ffff.ffff.ffff.ffff.ffff';
function getRangeValues(node) {
    const rangeQuery = utils_1.parseRange(node);
    const incMin = rangeQuery.gte != null;
    const incMax = rangeQuery.lte != null;
    const minValue = rangeQuery.gte || rangeQuery.gt || '*';
    const maxValue = rangeQuery.lte || rangeQuery.lt || '*';
    return {
        incMin, incMax, minValue, maxValue
    };
}
function ipTerm(node) {
    const argeCidr = is_cidr_1.default(node.value);
    if (argeCidr > 0) {
        const range = ip6addr_1.default.createCIDR(node.value);
        return pRangeTerm(range);
    }
    return function isIpterm(ip) {
        if (is_cidr_1.default(ip) > 0) {
            const argRange = ip6addr_1.default.createCIDR(ip);
            return argRange.contains(node.value);
        }
        return ip === node.value;
    };
}
exports.ipTerm = ipTerm;
function validateIpRange(node) {
    const values = getRangeValues(node);
    const { incMin, incMax } = values;
    let { minValue, maxValue } = values;
    if (utils_1.isInfiniteMin(minValue)) {
        net_1.isIPv6(maxValue) ? (minValue = MIN_IPV6_IP) : (minValue = MIN_IPV4_IP);
    }
    if (utils_1.isInfiniteMax(maxValue)) {
        net_1.isIPv6(minValue) ? (maxValue = MAX_IPV6_IP) : (maxValue = MAX_IPV4_IP);
    }
    if (!incMin) {
        minValue = ip6addr_1.default
            .parse(minValue)
            .offset(1)
            .toString();
    }
    if (!incMax) {
        maxValue = ip6addr_1.default
            .parse(maxValue)
            .offset(-1)
            .toString();
    }
    return { minValue, maxValue };
}
function checkCidr(ip, range) {
    const argRange = ip6addr_1.default.createCIDR(ip);
    return (range.contains(argRange.first().toString())
        || range.contains(argRange.last().toString())
        || argRange.contains(range.first().toString())
        || argRange.contains(range.last().toString()));
}
function pRangeTerm(range) {
    return function checkIp(ip) {
        if (is_cidr_1.default(ip) > 0) {
            return checkCidr(ip, range);
        }
        if (net_1.isIP(ip) > 0)
            return range.contains(ip);
        return false;
    };
}
function ipRange(node) {
    const { minValue, maxValue } = validateIpRange(node);
    const range = ip6addr_1.default.createAddrRange(minValue, maxValue);
    return pRangeTerm(range);
}
exports.ipRange = ipRange;
//# sourceMappingURL=ip.js.map