"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@terascope/utils");
function regexp(term) {
    return function regexpTerm(str) {
        if (typeof str !== 'string')
            return false;
        return matchString(str, term);
    };
}
exports.regexp = regexp;
function wildcard(term) {
    const wildCardQuery = parseWildCard(term);
    return function wildcardTerm(str) {
        if (typeof str !== 'string')
            return false;
        return matchString(str, wildCardQuery);
    };
}
exports.wildcard = wildcard;
function findWildcardField(field, cb) {
    return function WildcardField(data) {
        const resultsArray = recurseDownObject(field || '', data);
        if (resultsArray.length === 0)
            return false;
        return resultsArray.some(cb);
    };
}
exports.findWildcardField = findWildcardField;
function parseWildCard(term) {
    return term
        .replace('.', '\\.{0,1}')
        .replace(/\*/g, '.*')
        .replace(/\?/g, '[^\\n\\r\\s]');
}
exports.parseWildCard = parseWildCard;
function isWildCardString(term) {
    if (typeof term === 'string') {
        if (term.match('[?+*+]'))
            return true;
    }
    return false;
}
exports.isWildCardString = isWildCardString;
function matchString(field, wildCardQuery) {
    const regex = new RegExp(`^${wildCardQuery}$`);
    return typeof field === 'string' && field.match(regex) != null;
}
exports.matchString = matchString;
// city.*   city.deeper.*   city.*.*
function recurseDownObject(field, object) {
    const fieldSequence = field.split('.').map(parseWildCard);
    return recurse(fieldSequence, object);
}
function recurse(arr, obj) {
    if (arr.length === 0)
        return [];
    const field = arr.shift();
    const results = [];
    for (const [key, value] of Object.entries(obj)) {
        if (matchString(key, field)) {
            if (arr.length === 0) {
                results.push(value);
            }
            else if (utils_1.isPlainObject(value)) {
                results.push(...recurse(arr.slice(), value));
            }
        }
    }
    return results;
}
//# sourceMappingURL=string.js.map