"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
const utils_1 = require("../../../utils");
function validate(params) {
    const topLeftParam = params.find((node) => node.field === 'top_left');
    const bottomRightParam = params.find((node) => node.field === 'bottom_right');
    if (topLeftParam == null)
        throw new Error('Invalid geoBox query, need to specify a "topLeft" parameter');
    if (bottomRightParam == null)
        throw new Error('Invalid geoBox query, need to specify a "bottomRight" parameter');
    return {
        top_left: utils_1.parseGeoPoint(topLeftParam.value),
        bottom_right: utils_1.parseGeoPoint(bottomRightParam.value)
    };
}
const geoBox = {
    name: 'geoBox',
    version: '1',
    create(_field, params, { logger }) {
        if (!_field || _field === '*')
            throw new Error('Field for geoBox cannot be empty or "*"');
        // eslint-disable-next-line @typescript-eslint/camelcase
        const { top_left, bottom_right } = validate(params);
        function toElasticsearchQuery(field) {
            const query = {};
            query.geo_bounding_box = {};
            query.geo_bounding_box[field] = {
                top_left,
                bottom_right,
            };
            if (logger.level() === 10)
                logger.trace('built geo bounding box query', { query });
            return { query };
        }
        function matcher() {
            const polygon = helpers_1.makeBBox(top_left, bottom_right);
            // Nothing matches so return false
            if (polygon == null)
                return () => false;
            return helpers_1.polyHasPoint(polygon);
        }
        return {
            match: matcher(),
            toElasticsearchQuery
        };
    }
};
exports.default = geoBox;
//# sourceMappingURL=box.js.map