"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("@turf/helpers");
const helpers_2 = require("./helpers");
const utils_1 = require("../../../utils");
const interfaces_1 = require("../../../translator/interfaces");
const interfaces_2 = require("../../../interfaces");
function validate(params) {
    const geoPointParam = params.find((node) => node.field === 'point');
    if (geoPointParam == null)
        throw new Error('Invalid geoContainsPoint query, need to specify a "point" parameter');
    const pointData = utils_1.parseGeoPoint(geoPointParam.value);
    return { lat: pointData.lat, lon: pointData.lon };
}
const geoContainsPoint = {
    name: 'geoContainsPoint',
    version: '1',
    create(_field, params, { logger }) {
        if (!_field || _field === '*')
            throw new Error('Field for geoContainsPoint cannot be empty or "*"');
        const { lat, lon } = validate(params);
        function toElasticsearchQuery(field) {
            const query = {
                geo_shape: {
                    [field]: {
                        shape: {
                            type: interfaces_1.ESGeoShapeType.Point,
                            coordinates: [lon, lat]
                        },
                        relation: interfaces_2.GeoShapeRelation.Intersects
                    }
                }
            };
            if (logger.level() === 10)
                logger.trace('built geo shape query', { query });
            return { query };
        }
        function matcher() {
            const turfPoint = helpers_1.point([lon, lat]);
            return helpers_2.pointInGeoShape(turfPoint);
        }
        return {
            match: matcher(),
            toElasticsearchQuery
        };
    }
};
exports.default = geoContainsPoint;
//# sourceMappingURL=contains-point.js.map