"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const helpers_1 = require("./helpers");
const utils_1 = require("../../../utils");
function validate(params) {
    const distanceParam = params.find((node) => node.field === 'distance');
    const geoPointParam = params.find((node) => node.field === 'point');
    if (distanceParam == null)
        throw new Error('Invalid geoDistance query, need to specify a "distance" parameter');
    if (geoPointParam == null)
        throw new Error('Invalid geoDistance query, need to specify a "point" parameter');
    const point = utils_1.parseGeoPoint(geoPointParam.value);
    const distance = utils_1.parseGeoDistance(distanceParam.value);
    return {
        ...point,
        ...distance
    };
}
const geoDistance = {
    name: 'geoDistance',
    version: '1',
    create(_field, params, { logger }) {
        if (!_field || _field === '*')
            throw new Error('Field for geoDistance cannot be empty or "*"');
        const { lat, lon, distance, unit: paramUnit } = validate(params);
        function toElasticsearchQuery(field, options) {
            const unit = paramUnit || options.geo_sort_unit;
            const order = options.geo_sort_order;
            const query = {};
            query.geo_distance = {
                distance: `${distance}${unit}`,
            };
            query.geo_distance[field] = {
                lat,
                lon,
            };
            const sort = {
                _geo_distance: {
                    order,
                    unit,
                    [field]: {
                        lat,
                        lon
                    }
                }
            };
            if (logger.level() === 10)
                logger.trace('built geo distance query', { query });
            return {
                query,
                sort
            };
        }
        function matcher() {
            // There is a mismatch between elasticsearch and turf on "inch" naming
            const units = paramUnit === 'inch' ? 'inches' : paramUnit;
            const config = { units };
            const polygon = helpers_1.makeCircle({ lat, lon }, distance, config);
            // Nothing matches so return false
            if (polygon == null)
                return () => false;
            return helpers_1.polyHasPoint(polygon);
        }
        return {
            match: matcher(),
            toElasticsearchQuery
        };
    }
};
exports.default = geoDistance;
//# sourceMappingURL=distance.js.map