"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
const bbox_1 = __importDefault(require("@turf/bbox"));
const bbox_polygon_1 = __importDefault(require("@turf/bbox-polygon"));
// @ts-ignore
const boolean_equal_1 = __importDefault(require("@turf/boolean-equal"));
// @ts-ignore
const circle_1 = __importDefault(require("@turf/circle"));
const boolean_point_in_polygon_1 = __importDefault(require("@turf/boolean-point-in-polygon"));
// @ts-ignore
const boolean_within_1 = __importDefault(require("@turf/boolean-within"));
const boolean_contains_1 = __importDefault(require("@turf/boolean-contains"));
const boolean_disjoint_1 = __importDefault(require("@turf/boolean-disjoint"));
// @ts-ignore
const boolean_overlap_1 = __importDefault(require("@turf/boolean-overlap"));
const helpers_1 = require("@turf/helpers");
// @ts-ignore
const line_to_polygon_1 = __importDefault(require("@turf/line-to-polygon"));
const invariant_1 = require("@turf/invariant");
const utils_1 = require("../../../utils");
const interfaces_1 = require("../../../interfaces");
const interfaces_2 = require("../../../translator/interfaces");
function polyHasPoint(polygon) {
    return (fieldData) => {
        const point = utils_1.parseGeoPoint(fieldData, false);
        if (!point)
            return false;
        return boolean_point_in_polygon_1.default(makeCoordinatesFromGeoPoint(point), polygon);
    };
}
exports.polyHasPoint = polyHasPoint;
function makeCircle(point, distance, config) {
    return circle_1.default(makeCoordinatesFromGeoPoint(point), distance, config);
}
exports.makeCircle = makeCircle;
function makeBBox(point1, point2) {
    const line = helpers_1.lineString([
        makeCoordinatesFromGeoPoint(point1),
        makeCoordinatesFromGeoPoint(point2)
    ]);
    const box = bbox_1.default(line);
    return bbox_polygon_1.default(box);
}
exports.makeBBox = makeBBox;
function pointInGeoShape(searchPoint) {
    return (fieldData) => {
        let polygon;
        if (isGeoShapePoint(fieldData)) {
            return boolean_equal_1.default(searchPoint, helpers_1.point(fieldData.coordinates));
        }
        if (isGeoShapeMultiPolygon(fieldData)) {
            polygon = helpers_1.multiPolygon(fieldData.coordinates);
        }
        if (isGeoShapePolygon(fieldData)) {
            polygon = helpers_1.polygon(fieldData.coordinates);
        }
        // Nothing matches so return false
        if (!polygon)
            return false;
        return boolean_point_in_polygon_1.default(searchPoint, polygon);
    };
}
exports.pointInGeoShape = pointInGeoShape;
function makeShape(geoShape) {
    let feature;
    if (isGeoShapePoint(geoShape)) {
        return helpers_1.point(geoShape.coordinates);
    }
    if (isGeoShapeMultiPolygon(geoShape)) {
        return helpers_1.multiPolygon(geoShape.coordinates);
    }
    if (isGeoShapePolygon(geoShape)) {
        return helpers_1.polygon(geoShape.coordinates);
    }
    return feature;
}
exports.makeShape = makeShape;
function validateListCoords(coords) {
    if (coords.length < 3)
        throw new Error('Points parameter for a geoPolygon query must have at least three geo-points');
    const line = helpers_1.lineString(coords);
    const polygon = line_to_polygon_1.default(line);
    return invariant_1.getCoords(polygon);
}
exports.validateListCoords = validateListCoords;
function polyHasShape(queryPolygon, relation) {
    const match = getRelationFn(relation, queryPolygon);
    return (fieldData) => {
        const feature = makeShape(fieldData);
        // Nothing matches so return false
        if (!feature)
            return false;
        try {
            return match(feature);
        }
        catch (err) {
            return false;
        }
    };
}
exports.polyHasShape = polyHasShape;
// within returns true if the first geometry is completely within the second geometry
function withinFn(queryPolygon) {
    return (fieldPolygon) => boolean_within_1.default(fieldPolygon, queryPolygon);
}
// contains returns True if the second geometry is completely contained by the first geometry.
function containsFn(queryPolygon) {
    return (fieldPolygon) => boolean_contains_1.default(fieldPolygon, queryPolygon);
}
// Compares two geometries of the same dimension and returns true if they intersection
function intersectFn(queryPolygon) {
    return (fieldPolygon) => boolean_overlap_1.default(fieldPolygon, queryPolygon);
}
// disjoint returns (TRUE) if the intersection of the two geometries is an empty set.
function disjointFn(queryPolygon) {
    return (fieldPolygon) => boolean_disjoint_1.default(fieldPolygon, queryPolygon);
}
const relationOptions = {
    [interfaces_1.GeoShapeRelation.Within]: withinFn,
    [interfaces_1.GeoShapeRelation.Disjoint]: disjointFn,
    [interfaces_1.GeoShapeRelation.Contains]: containsFn,
    [interfaces_1.GeoShapeRelation.Intersects]: intersectFn,
};
function getRelationFn(relation, queryPolygon) {
    return relationOptions[relation](queryPolygon);
}
exports.getRelationFn = getRelationFn;
function makeCoordinatesFromGeoPoint(point) {
    return [point.lon, point.lat];
}
exports.makeCoordinatesFromGeoPoint = makeCoordinatesFromGeoPoint;
function isGeoJSONData(input) {
    return input.coordinates != null
        && Array.isArray(input.coordinates)
        && input.type != null;
}
exports.isGeoJSONData = isGeoJSONData;
function isGeoShapePoint(shape) {
    return shape.type === interfaces_1.GeoShapeType.Point || shape.type === interfaces_2.ESGeoShapeType.Point;
}
exports.isGeoShapePoint = isGeoShapePoint;
function isGeoShapePolygon(shape) {
    return shape.type === interfaces_1.GeoShapeType.Polygon || shape.type === interfaces_2.ESGeoShapeType.Polygon;
}
exports.isGeoShapePolygon = isGeoShapePolygon;
function isGeoShapeMultiPolygon(shape) {
    return shape.type === interfaces_1.GeoShapeType.MultiPolygon || shape.type === interfaces_2.ESGeoShapeType.MultiPolygon;
}
exports.isGeoShapeMultiPolygon = isGeoShapeMultiPolygon;
//# sourceMappingURL=helpers.js.map