"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@terascope/utils");
const peg_engine_1 = require("./peg-engine");
const i = __importStar(require("./interfaces"));
const utils = __importStar(require("./utils"));
const _logger = utils_1.debugLogger('xlucene-parser');
class Parser {
    constructor(query, options = {}) {
        this.logger = options.logger != null
            ? options.logger.child({ module: 'xlucene-parser' })
            : _logger;
        this.query = utils_1.trim(query || '');
        this.variables = options.variables ? utils.validateVariables(options.variables) : {};
        const contextArg = {
            typeConfig: options.type_config,
            variables: this.variables,
            logger: this.logger
        };
        try {
            this.ast = peg_engine_1.parse(this.query, { contextArg });
            this.forTypes([i.ASTType.Function], (_node) => {
                const node = _node;
                // @ts-ignore we are delaying instantiation
                if (node.instance)
                    node.instance = node.instance();
            });
            if (this.logger.level() === 10) {
                const astJSON = JSON.stringify(this.ast, null, 4);
                this.logger.trace(`parsed ${this.query ? this.query : "''"} to `, astJSON);
            }
        }
        catch (err) {
            if (err && err.message.includes('Expected ,')) {
                err.message = err.message.replace('Expected ,', 'Expected');
            }
            throw new utils_1.TSError(err, {
                reason: `Failure to parse xlucene query "${this.query}"`,
            });
        }
    }
    forTypes(types, cb) {
        const walkNode = (node) => {
            if (types.includes(node.type)) {
                cb(node);
            }
            if (utils.isNegation(node)) {
                walkNode(node.node);
                return;
            }
            if (utils.isGroupLike(node)) {
                for (const conj of node.flow) {
                    walkNode(conj);
                }
                return;
            }
            if (utils.isConjunction(node)) {
                for (const conj of node.nodes) {
                    walkNode(conj);
                }
            }
        };
        walkNode(this.ast);
    }
    forTermTypes(cb) {
        this.forTypes(utils.termTypes, cb);
    }
}
exports.Parser = Parser;
//# sourceMappingURL=parser.js.map