"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@terascope/utils");
const i = __importStar(require("./interfaces"));
const interfaces_1 = require("../interfaces");
const string_1 = require("../document-matcher/logic-builder/string");
function isLogicalGroup(node) {
    return node && node.type === i.ASTType.LogicalGroup;
}
exports.isLogicalGroup = isLogicalGroup;
function isConjunction(node) {
    return node && node.type === i.ASTType.Conjunction;
}
exports.isConjunction = isConjunction;
function isNegation(node) {
    return node && node.type === i.ASTType.Negation;
}
exports.isNegation = isNegation;
function isFieldGroup(node) {
    return node && node.type === i.ASTType.FieldGroup;
}
exports.isFieldGroup = isFieldGroup;
function isExists(node) {
    return node && node.type === i.ASTType.Exists;
}
exports.isExists = isExists;
function isRange(node) {
    return node && node.type === i.ASTType.Range;
}
exports.isRange = isRange;
function isGeoDistance(node) {
    return node && node.type === i.ASTType.GeoDistance;
}
exports.isGeoDistance = isGeoDistance;
function isGeoBoundingBox(node) {
    return node && node.type === i.ASTType.GeoBoundingBox;
}
exports.isGeoBoundingBox = isGeoBoundingBox;
function isFunctionExpression(node) {
    return node && node.type === i.ASTType.Function;
}
exports.isFunctionExpression = isFunctionExpression;
function isRegexp(node) {
    return node && node.type === i.ASTType.Regexp;
}
exports.isRegexp = isRegexp;
function isWildcard(node) {
    return node && node.type === i.ASTType.Wildcard;
}
exports.isWildcard = isWildcard;
function isWildcardField(node) {
    return node && string_1.isWildCardString(node.field);
}
exports.isWildcardField = isWildcardField;
function isTerm(node) {
    return node && node.type === i.ASTType.Term;
}
exports.isTerm = isTerm;
function isEmptyAST(node) {
    return utils_1.isEmpty(node) || (node && node.type === i.ASTType.Empty);
}
exports.isEmptyAST = isEmptyAST;
function isStringDataType(node) {
    return node && node.field_type === 'string';
}
exports.isStringDataType = isStringDataType;
exports.numberDataTypes = [interfaces_1.FieldType.Integer, interfaces_1.FieldType.Float];
function isNumberDataType(node) {
    return node && exports.numberDataTypes.includes(node.field_type);
}
exports.isNumberDataType = isNumberDataType;
function isBooleanDataType(node) {
    return node && node.field_type === 'boolean';
}
exports.isBooleanDataType = isBooleanDataType;
function getAnyValue(node) {
    return node && node.value;
}
exports.getAnyValue = getAnyValue;
function getField(node) {
    if (!node)
        return;
    if (!node.field || !utils_1.isString(node.field))
        return;
    return node.field;
}
exports.getField = getField;
/** term level queries with field (string|null)  */
exports.termTypes = [
    i.ASTType.Term,
    i.ASTType.Regexp,
    i.ASTType.Range,
    i.ASTType.Wildcard,
    i.ASTType.GeoDistance,
    i.ASTType.GeoBoundingBox,
    i.ASTType.Function
];
function isTermType(node) {
    return node && exports.termTypes.includes(node.type);
}
exports.isTermType = isTermType;
/** logical group or field group with flow */
exports.groupTypes = [i.ASTType.LogicalGroup, i.ASTType.FieldGroup];
function isGroupLike(node) {
    return node && exports.groupTypes.includes(node.type);
}
exports.isGroupLike = isGroupLike;
function validateVariables(obj) {
    if (!utils_1.isPlainObject(obj))
        throw new utils_1.TSError('Invalid variables configuration provided, it must be an object');
    return { ...obj };
}
exports.validateVariables = validateVariables;
//# sourceMappingURL=utils.js.map