import * as es from 'elasticsearch';
import * as ts from '@terascope/utils';
import { SortOrder } from '../translator';
import * as i from './interfaces';
import { GeoDistanceUnit, TypeConfig, Variables } from '../interfaces';
export declare class QueryAccess<T extends ts.AnyObject = ts.AnyObject> {
    readonly excludes: (keyof T)[];
    readonly includes: (keyof T)[];
    readonly constraints?: string[];
    readonly preventPrefixWildcard: boolean;
    readonly allowImplicitQueries: boolean;
    readonly defaultGeoField?: string;
    readonly defaultGeoSortOrder?: SortOrder;
    readonly defaultGeoSortUnit?: GeoDistanceUnit | string;
    readonly allowEmpty: boolean;
    readonly typeConfig: TypeConfig;
    readonly parsedTypeConfig: TypeConfig;
    readonly variables: Variables;
    logger: ts.Logger;
    private readonly _parser;
    private readonly _translator;
    constructor(config?: i.QueryAccessConfig<T>, options?: i.QueryAccessOptions);
    clearCache(): void;
    /**
     * Validate and restrict a xlucene query
     *
     * @returns a restricted xlucene query
     */
    restrict(q: string, options?: i.RestrictOptions): string;
    /**
     * Validate and restrict a xlucene query
     *
     * @returns a restricted xlucene query
     */
    private _restrict;
    private _restrictTypeConfig;
    /**
     * Converts a restricted xlucene query to an elasticsearch search query
     *
     * @returns a restricted elasticsearch search query
     */
    restrictSearchQuery(query: string, opts?: i.RestrictSearchQueryOptions): Promise<es.SearchParams>;
    /**
     * Restrict requested source to all or subset of the ones available
     *
     * **NOTE:** this will remove restricted fields and will not throw
     */
    restrictSourceFields(includes?: (keyof T)[], excludes?: (keyof T)[]): {
        includes: (keyof T)[] | undefined;
        excludes: (keyof T)[] | undefined;
    };
    private _getSourceFields;
    private _isFieldRestricted;
    private _addConstraints;
}
//# sourceMappingURL=query-access.d.ts.map