"use strict";
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (Object.hasOwnProperty.call(mod, k)) result[k] = mod[k];
    result["default"] = mod;
    return result;
};
Object.defineProperty(exports, "__esModule", { value: true });
const utils_1 = require("@terascope/utils");
const parser_1 = require("../parser");
const utils_2 = require("../utils");
const utils = __importStar(require("./utils"));
const _logger = utils_1.debugLogger('xlucene-translator');
class Translator {
    constructor(input, options = {}) {
        this._defaultGeoSortOrder = 'asc';
        this._defaultGeoSortUnit = 'meters';
        this.variables = options.variables;
        this.logger = options.logger != null
            ? options.logger.child({ module: 'xlucene-translator' })
            : _logger;
        this.typeConfig = options.type_config || {};
        if (utils_1.isString(input)) {
            this._parser = new parser_1.Parser(input, {
                type_config: this.typeConfig,
                logger: this.logger,
                variables: this.variables
            });
        }
        else {
            this._parser = input;
        }
        if (options.default_geo_field) {
            this._defaultGeoField = options.default_geo_field;
        }
        if (options.default_geo_sort_order) {
            this._defaultGeoSortOrder = options.default_geo_sort_order;
        }
        if (options.default_geo_sort_unit) {
            this._defaultGeoSortUnit = utils_2.parseGeoDistanceUnit(options.default_geo_sort_unit);
        }
        this.query = this._parser.query;
    }
    toElasticsearchDSL(opts = {}) {
        const result = utils.translateQuery(this._parser, {
            logger: this.logger,
            type_config: this.typeConfig,
            default_geo_field: this._defaultGeoField,
            geo_sort_point: opts.geo_sort_point,
            geo_sort_order: opts.geo_sort_order || this._defaultGeoSortOrder,
            geo_sort_unit: opts.geo_sort_unit || this._defaultGeoSortUnit,
        });
        if (this.logger.level() === 10) {
            const resultStr = JSON.stringify(result, null, 2);
            this.logger.trace(`translated ${this.query ? this.query : "''"} query to`, resultStr);
        }
        return result;
    }
}
exports.Translator = Translator;
//# sourceMappingURL=translator.js.map